<?php
require_once '../include/utils.php';

if (!isset($_GET['id']) || !ctype_digit($_GET['id'])) {
	die_with_message(
		'Invalid request',
		'ID is not specified or illegal.'
	);
}

$id = $_GET['id'];
$artwork = json_decode(
	get(sprintf(ARTWORK_INFO_URL, $id))
)->body;

if (!$artwork) {
	die_with_message(
		'Not Found',
		'Unable to find that artwork.'
	);
}

if (artwork_is_hidden($artwork)) {
	die_with_message(
		'Artwork Hidden',
		"Unable to show that artwork due to access control on restricted content."
	);
}

render_page_header(htmlspecialchars($artwork->title));
?>

<div class="container">
	<?php render_artwork_images($artwork) ?>

	<h1><?= htmlspecialchars($artwork->title) ?></h1>

	<p><?= $artwork->description ?></p>

	<p><div class="tags">
		<?php foreach ($artwork->tags->tags as $tag) {
			$tag_ja = htmlspecialchars($tag->tag);
			$tag_en = htmlspecialchars(
				isset($tag->translation->en) ? $tag->translation->en
				: (isset($tag->romaji) ? $tag->romaji : '')
			);
			?>
			<div>
				<a href="<?= BASE_URL ?>/tags/<?= $tag_ja ?>">#<?= $tag_ja ?></a>
				<small><?= $tag_en ?></small>
			</div>
		<?php } ?>
	</div></p>

	<p><div class="stats">
		<div>Likes: <?= $artwork->likeCount ?></div>
		<div>Bookmarks: <?= $artwork->bookmarkCount ?></div>
		<div>Views: <?= $artwork->viewCount ?></div>
	</div></p>

	<p><?= gmdate('Y-m-d H:i:s', strtotime($artwork->createDate)) . ' UTC'; ?>

	<?php
	$pfp_url = '';
	foreach ($artwork->userIllusts as $v) {
		if (isset($v->profileImageUrl)) {
			$pfp_url = get_proxy_url($v->profileImageUrl);
		}
	}
	?>

	<p><a class="img-with-text" href="<?= BASE_URL ?>/users/<?= $artwork->userId ?>">
		<img class="avatar" src="<?= $pfp_url ?>">
		<span><?= htmlspecialchars($artwork->userName) ?></span>
	</a></p>

	<hr>

	<h2>Comments (<?= $artwork->commentCount ?>)</h2>
	<?php
	if ($artwork->commentCount == 0) {
		echo '<p>No comments.</p>';
	}
	else {
		$comments = json_decode(
			get(sprintf(ARTWORK_COMMENTS_URL, $id, COMMENTS_LIMIT))
		)->body->comments;
		render_comment_list($comments);
	}
	?>
</div>

<div class="container container-big">
	<hr>
	<h2>Recommended</h2>
	<?php
	$recommended = json_decode(
		get(sprintf(ARTWORK_RECOMMENDED_URL, $id, RECOMMEND_LIMIT))
	)->body->illusts;
	render_artworks_grid($recommended);
	?>
</div>

<?php
render_page_footer();
