<?php
require_once 'config.php';
require_once 'user_settings.php';

load_settings($_COOKIE);

const DISCOVERY_URL = 'https://www.pixiv.net/ajax/discovery/artworks?mode=all&limit=%s';
const ARTWORK_INFO_URL = 'https://www.pixiv.net/ajax/illust/%s';
const ARTWORK_COMMENTS_URL = 'https://www.pixiv.net/ajax/illusts/comments/roots?illust_id=%s&limit=%s';
const ARTWORK_RECOMMENDED_URL = 'https://www.pixiv.net/ajax/illust/%s/recommend/init?limit=%s';
const USER_INFO_URL = 'https://www.pixiv.net/ajax/user/%s?full=1';
const USER_ARTWORKS_URL = 'https://www.pixiv.net/ajax/user/%s/profile/all';
const USER_ARTWORKS_INFO_URL = 'https://www.pixiv.net/ajax/user/%s/profile/illusts?work_category=illustManga&is_first_page=0&%s';
const SEARCH_ARTWORKS_URL = 'https://www.pixiv.net/ajax/search/artworks/%s?p=%s';
const SEARCH_TAG_URL = 'https://www.pixiv.net/ajax/search/tags/%s';
const IMG_BASE_URL = 'https://i.pximg.net/';
const STATIC_BASE_URL = 'https://s.pximg.net/';

const ARTWORKS_PER_PAGE = 24;
const PAGINATOR_RADIUS = 5;
const COMMENTS_FIRST_PAGE = 5;
const COMMENTS_PER_PAGE = 20;

const EMOJI_IDS = [
	'normal' => '101',
	'surprise' => '102',
	'serious' => '103',
	'heaven' => '104',
	'happy' => '105',
	'excited' => '106',
	'sing' => '107',
	'cry' => '108',
	'normal2' => '201',
	'shame2' => '202',
	'love2' => '203',
	'interesting2' => '204',
	'blush2' => '205',
	'fire2' => '206',
	'angry2' => '207',
	'shine2' => '208',
	'panic2' => '209',
	'normal3' => '301',
	'satisfaction3' => '302',
	'surprise3' => '303',
	'smile3' => '304',
	'shock3' => '305',
	'gaze3' => '306',
	'wink3' => '307',
	'happy3' => '308',
	'excited3' => '309',
	'love3' => '310',
	'normal4' => '401',
	'surprise4' => '402',
	'serious4' => '403',
	'love4' => '404',
	'shine4' => '405',
	'sweat4' => '406',
	'shame4' => '407',
	'sleep4' => '408',
	'heart' => '501',
	'teardrop' => '502',
	'star' => '503'
];

function get(string $url, &$response_headers = []) {
	$stream_context = [
		'http' => [
			'header' =>
				"Cookie: PHPSESSID=" . SESSION_ID . PHP_EOL .
				"User-Agent: Mozilla/5.0" . PHP_EOL .
				'Referer: https://www.pixiv.net/'
		]
	];
	if (PROXY) $stream_context['http']['proxy'] = 'tcp://' . PROXY;
	$response = file_get_contents($url, false, stream_context_create($stream_context));
	$response_headers = $http_response_header;
	if (strpos($response_headers[0], '200 OK') !== false) return $response;
	return false;
}

function get_proxy_url($url) {
	return BASE_URL . '/proxy.php?url=' . $url;
}

function artwork_is_hidden($artwork) {
	global $settings;
	return (
		($artwork->sl >= 4 && (SSFW_ONLY || $settings['show-r15'] == 'off'))
		|| ($artwork->xRestrict == 1 && (SFW_ONLY || $settings['show-r18'] == 'off'))
		|| ($artwork->xRestrict == 2 && (SFW_ONLY || $settings['show-r18g'] == 'off'))
	);
}

function censor($artworks) {
	$result = [];
	foreach ($artworks as $artwork) {
		if (!artwork_is_hidden($artwork)) array_push($result, $artwork);
	}
	return $result;
}

function render_page_header($page_title) {
	require_once 'header.php';
}

function render_page_footer() {
	require_once 'footer.php';
}

function die_with_message($title, $message) {
	render_page_header($title, $message); ?>
	<div class="container container-small">
		<h1><?= $title ?></h1>
		<p><?= $message ?></p>
	</div><?php
	render_page_footer();
	die;
}

function parse_comment($comment) {
	return preg_replace_callback('/\((.+?)\)/', function($matches) {
		if(isset(EMOJI_IDS[$matches[1]])) {
			$emoji_url = get_proxy_url(STATIC_BASE_URL . 'common/images/emoji/' . EMOJI_IDS[$matches[1]] . '.png');
			return "<img class='emoji' src='{$emoji_url}'>";
		}
		else return $matches[0];
	}, $comment);
}

function render_collapsed_content($page_count, $button_text, callable $render_page) {
	for ($page = 0; $page < $page_count; $page ++) {
		if ($page > 0) echo '<details><summary>' . $button_text .'</summary>';
		call_user_func($render_page, $page);
	}
	echo str_repeat('</details>', $page_count - 1);
}

function render_artworks_grid($data_uncencored, $collapsed = true) {
	$data = censor($data_uncencored);
	if (!$data) {
		if ($data_uncencored) echo '<p>No work can be shown due to access control on restircted content.';
		else echo '<p>No works to show.</p>';
		return;
	}
	$page_count = $collapsed ? ceil(count($data) / ARTWORKS_PER_PAGE) : 1;
	render_collapsed_content($page_count, 'Load More', function($page) use ($data, $collapsed) {
		$artworks_in_page = $collapsed ? array_slice($data, $page * ARTWORKS_PER_PAGE, ARTWORKS_PER_PAGE) : $data; ?>
		<div class="grid">
			<?php foreach ($artworks_in_page as $artwork) { ?>
				<a href="<?= BASE_URL ?>/artworks/<?= $artwork->id ?>">
					<div class="extra-info"><?php
						if ($artwork->xRestrict != 0) { ?>
							<div class="content-warning"><?= [1 => 'R-18', 2 => 'R-18G'][$artwork->xRestrict] ?></div><?php
						}
						if ($artwork->pageCount > 1) { ?>
							<div class="page-count">🗍 <?= $artwork->pageCount ?></div><?php
						} ?>
					</div>
					<img loading="lazy" src="<?= get_proxy_url($artwork->url) ?>" alt="<?= htmlspecialchars($artwork->alt); ?>">
					<div><?= htmlspecialchars($artwork->title); ?></div>
				</a>
			<?php } ?>
		</div><?php
	});
}

function render_artwork_images($artwork) { ?>
	<div class="artwork-images"><?php
		render_collapsed_content($artwork->pageCount > 1 ? 2 : 1, 'Show All', function(bool $remaining) use ($artwork) {
			for ($p = ($remaining ? 1 : 0); $p < ($remaining ? $artwork->pageCount : 1); $p ++) { ?>
				<a target="_blank" href="<?= get_proxy_url(str_replace('_p0', '_p' . $p, $artwork->urls->original)) ?>">
					<img alt="Page <?= $p + 1 ?>" src="<?= get_proxy_url(str_replace('_p0', '_p' . $p, $artwork->urls->regular)) ?>">
				</a><?php
			}
		}); ?>
	</div><?php
}

function render_comment_list($comments) {
	if (!$comments) {
		echo '<p>Failed to fetch comments</p>';
		return;
	}
	$page_count = ceil((count($comments) - COMMENTS_FIRST_PAGE)/ COMMENTS_PER_PAGE) + 1;
	render_collapsed_content($page_count, 'Show More', function($page) use ($comments) {
		$comments_in_page = array_slice(
			$comments,
			$page == 0 ? 0 : ($page - 1) * COMMENTS_PER_PAGE + COMMENTS_FIRST_PAGE,
			$page == 0 ? COMMENTS_FIRST_PAGE : COMMENTS_PER_PAGE
		);
		foreach ($comments_in_page as $comment) { ?>
			<div class="comment">
				<div class="comment-left">
					<a href="<?= BASE_URL ?>/users/<?= $comment->commentUserId ?>">
						<img class="avatar" src="<?= get_proxy_url($comment->img) ?>">
					</a>
				</div>
				<div class="comment-right">
					<div class="comment-info">
						<a href="<?= BASE_URL ?>/users/<?= $comment->commentUserId ?>"><?= htmlspecialchars($comment->userName) ?></a>
						<small><?= $comment->commentDate ?></small>
					</div>
					<p class="comment-content"><?php
						if ($comment->comment) {
							echo parse_comment($comment->comment);
						}
						else {
							$stamp_url = get_proxy_url(STATIC_BASE_URL . 'common/images/stamp/generated-stamps/' . $comment->stampId . '_s.jpg');
							echo "<img class='stamp' src='{$stamp_url}'>";
						} ?>
					</p>
				</div>
			</div><?php
		}
	});
}

function render_paginator($page_count, $current_page, callable $get_page_url) { ?>
	<div class="paginator"><?php
		if ($current_page > PAGINATOR_RADIUS) { ?>
			<a href="<?= call_user_func($get_page_url, 1) ?>">1</a>
			<span>···</span><?php
		}
		for (
			$i = max(1, $current_page - PAGINATOR_RADIUS + 1);
			$i <= min($current_page + PAGINATOR_RADIUS - 1, $page_count);
			$i ++
		) {
			if ($i == $current_page) {
				?><a><?= $i ?></a><?php
			}
			else {
				?><a href="<?= call_user_func($get_page_url, $i) ?>"><?= $i ?></a><?php
			}
		}
		if ($current_page < $page_count - PAGINATOR_RADIUS) {
			?><span>···</span>
			<a href="<?= call_user_func($get_page_url, $page_count) ?>"><?= $page_count ?></a><?php
		}
	?>
	</div><?php
}
