<?php
require_once '../include/utils.php';

if (!isset($_GET['id']) || !ctype_digit($_GET['id'])) {
	die_with_message(
		'Invalid request',
		'ID is not specified or illegal.'
	);
}

$id = $_GET['id'];
$user_info = json_decode(
	get(sprintf(USER_INFO_URL, $id))
)->body;

if (!$user_info) {
	die_with_message(
		'Not Found',
		'Unable to find that user.'
	);
}

$username = htmlspecialchars($user_info->name);

render_page_header($username . "'s artworks");
?>

<div class="container">
	<h1 class="img-with-text">
		<img class="avatar" src="<?= get_proxy_url($user_info->imageBig) ?>">
		<span><?= $username ?></span>
	</h1>

	<p><?= str_replace(PHP_EOL, "<br>", htmlspecialchars($user_info->comment)); ?></p>

	<hr>

	<?php
	$artworks = json_decode(
		get(sprintf(USER_ARTWORKS_URL, $id))
	, true)['body'];

	$artwork_ids = array_merge(
		array_keys($artworks['illusts']),
		array_keys($artworks['manga'])
	);
	rsort($artwork_ids);

	$artwork_count = count($artwork_ids);
	$page_count = ceil($artwork_count / ARTWORKS_PER_PAGE);
	$current_page = (
		isset($_GET['p']) && ctype_digit($_GET['p']) && $_GET['p'] > 0 && $_GET['p'] <= $page_count
	) ? $_GET['p'] : 1;
	$artwork_ids_in_page = array_slice($artwork_ids, ($current_page - 1) * ARTWORKS_PER_PAGE, ARTWORKS_PER_PAGE);

	$artworks_in_page = json_decode(
		get(sprintf(USER_ARTWORKS_INFO_URL, $id, http_build_query(['ids' => $artwork_ids_in_page])))
	)->body->works;
	?>

	<h2 id="artworks">Illustrations and Manga (<?= $artwork_count ?>)</h2>

	<?php
	render_artworks_grid($artworks_in_page, false);

	render_paginator($page_count, $current_page, function($page) use ($id) {
		return BASE_URL . '/users/' . $id . '?p=' . $page . '#artworks';
	});
	?>
</div>

<?php
render_page_footer();
